# -*- coding: utf-8 -*-
# Most imports are after python2/3 check further down
import configparser
import argparse
import os
import re
import time
import sys

from .breachcompilation import breachcomp_check
from .classes import target
from .colors import colors as c
from .helpers import (
    fetch_emails,
    find_files,
    get_config_from_file,
    get_emails_from_file,
    print_banner,
    save_results_csv,
    check_latest_version,
    check_scylla_online,
)
from .localsearch import local_search, local_search_single, local_to_targets
from .localgzipsearch import local_gzip_search, local_search_single_gzip
from .summary import print_summary
from .chase import chase
from .print_results import print_results
from .gen_config import gen_config_file
from .url import target_urls


def target_factory(targets, user_args):
    """
    Receives list of emails and user args. Fetchs API keys from config file using user_args path and cli keys.
    For each target, launch target.methods() associated to found config artifacts.
    Handles chase logic with counters from enumerate()
    """
    # Removing duplicates here to avoid dups from chasing
    targets = list(set(targets))

    finished = []
    if user_args.config_file is not None or user_args.cli_apikeys is not None:
        api_keys = get_config_from_file(user_args)
    else:
        api_keys = None
    init_targets_len = len(targets)

    query = "email"
    skip_default_queries = False
    if user_args.user_query is not None:
        query = user_args.user_query
        skip_default_queries = True  # custom query skips default query automatically

    scylla_up = False
    if user_args.skip_defaults is False:
        scylla_up = check_scylla_online()



    for counter, t in enumerate(targets):
        c.info_news("Target factory started for {target}".format(target=t))
        if user_args.debug:
            current_target = target(t, debug=True)
        else:
            current_target = target(t)
        if not skip_default_queries:
            if not user_args.skip_defaults:
                current_target.get_hunterio_public()
                ## emailrep seems to insta-block h8mail user agent without a key
                # if api_keys is None or "emailrep" not in api_keys:
                #     current_target.get_emailrepio()
                # elif (
                #     api_keys is not None and "emailrep" in api_keys and query == "email"
                # ):
                #     current_target.get_emailrepio(api_keys["emailrep"])

        if api_keys is not None:
            if "hibp" in api_keys and query == "email":
                current_target.get_hibp3(api_keys["hibp"])
            if "emailrep" in api_keys and query == "email":
                current_target.get_emailrepio(api_keys["emailrep"])
            if "hunterio" in api_keys and query == "email":
                current_target.get_hunterio_private(api_keys["hunterio"])
            if "intelx_key" in api_keys:
                current_target.get_intelx(api_keys)
            if "snusbase_token" in api_keys:
                if "snusbase_url" in api_keys:
                    snusbase_url = api_keys["snusbase_url"]
                else:
                    snusbase_url = "http://api.snusbase.com/v2/search"
                current_target.get_snusbase(
                    snusbase_url, api_keys["snusbase_token"], query
                )
            if "leak-lookup_priv" in api_keys:
                current_target.get_leaklookup_priv(api_keys["leak-lookup_priv"], query)
            if "leak-lookup_pub" in api_keys and query == "email":
                current_target.get_leaklookup_pub(api_keys["leak-lookup_pub"])
            if "weleakinfo_pub" in api_keys and query == "email":
                current_target.get_weleakinfo_pub(api_keys["weleakinfo_pub"])
            if "weleakinfo_priv" in api_keys:
                current_target.get_weleakinfo_priv(api_keys["weleakinfo_priv"], query)
            if "dehashed_key" in api_keys:
                if "dehashed_email" in api_keys:
                    current_target.get_dehashed(
                        api_keys["dehashed_email"], api_keys["dehashed_key"], query
                    )
                else:
                    c.bad_news("Missing Dehashed email")
        if scylla_up:
            current_target.get_scylla(query)

        # Chasing
        if user_args.chase_limit and counter <= init_targets_len:
            user_args_force_email = user_args
            user_args_force_email.user_query = "email"
            user_args_force_email.chase_limit -= 1
            finished_chased = target_factory(
                chase(current_target, user_args), user_args_force_email
            )
            finished.extend((finished_chased))
        finished.append(current_target)
    return finished


def h8mail(user_args):
    """
    Handles most user arg logic. Creates a list() of targets from user input.
    Starts the target object factory loop; starts local searches after factory if in user inputs
    Prints results, saves to csv if in user inputs
    """

    if user_args.user_targets and user_args.user_urls:
        c.bad_news("Cannot use --url with --target. Use one or the other.")
        exit(1)

    if not user_args.user_targets and not user_args.user_urls:
        c.bad_news("Missing Target or URL")
        exit(1)

    start_time = time.time()

    import warnings

    warnings.filterwarnings('ignore', message='Unverified HTTPS request')

    targets = []
    if user_args.user_urls:
        targets = target_urls(user_args)
        if len(targets) == 0:
            c.bad_news("No targets found in URLs. Quitting")
            exit(0)

    # If we found emails from URLs, `targets` array already has stuff
    if len(targets) != 0:
        if user_args.user_targets is None:
            user_args.user_targets = []
            user_args.user_targets.extend(targets)

    else:  # Find targets in user input or file
        if user_args.user_targets is not None:
            for arg in user_args.user_targets:
                user_stdin_target = fetch_emails(arg, user_args)
                if os.path.isfile(arg):
                    c.info_news("Reading from file " + arg)
                    targets.extend(get_emails_from_file(arg, user_args))
                elif user_stdin_target:
                    targets.extend(user_stdin_target)
                else:
                    c.bad_news("No targets found in user input. Quitting")
                    exit(0)

    c.info_news("Removing duplicates")
    targets = list(set(targets))

    c.good_news("Targets:")
    for t in targets:
        c.good_news(t)

    # Launch
    breached_targets = target_factory(targets, user_args)

    # These are not done inside the factory as the factory iterates over each target individually
    # The following functions perform line by line checks of all targets per line

    if user_args.bc_path:
        breached_targets = breachcomp_check(breached_targets, user_args.bc_path)

    local_found = None
    # Handle cleartext search
    if user_args.local_breach_src:
        for arg in user_args.local_breach_src:
            res = find_files(arg)
            if user_args.single_file:
                local_found = local_search_single(res, targets)
            else:
                local_found = local_search(res, targets)
            if local_found is not None:
                breached_targets = local_to_targets(
                    breached_targets, local_found, user_args
                )
    # Handle gzip search
    if user_args.local_gzip_src:
        for arg in user_args.local_gzip_src:
            res = find_files(arg, "gz")
            if user_args.single_file:
                local_found = local_search_single_gzip(res, targets)
            else:
                local_found = local_gzip_search(res, targets)
            if local_found is not None:
                breached_targets = local_to_targets(
                    breached_targets, local_found, user_args
                )

    print_results(breached_targets, user_args.hide)

    print_summary(start_time, breached_targets)
    if user_args.output_file:
        save_results_csv(user_args.output_file, breached_targets)


def parse_args(args):
    """
    Seperate functions to make it easier to run tests
    Pass args as an array
    """
    parser = argparse.ArgumentParser(
        description="Email information and password lookup tool", prog="h8mail"
    )

    parser.add_argument(
        "-t",
        "--targets",
        dest="user_targets",
        help="Either string inputs or files. Supports email pattern matching from input or file, filepath globing and multiple arguments",
        nargs="+",
    )
    parser.add_argument(
        "-u",
        "--url",
        dest="user_urls",
        help="Either string inputs or files. Supports URL pattern matching from input or file, filepath globing and multiple arguments. Parse URLs page for emails. Requires http:// or https:// in URL.",
        nargs="+",
    )
    parser.add_argument(
        "-q",
        "--custom-query",
        dest="user_query",
        help='Perform a custom query. Supports username, password, ip, hash, domain. Performs an implicit "loose" search when searching locally',
    )
    parser.add_argument(
        "--loose",
        dest="loose",
        help="Allow loose search by disabling email pattern recognition. Use spaces as pattern seperators",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "-c",
        "--config",
        dest="config_file",
        help="Configuration file for API keys. Accepts keys from Snusbase, WeLeakInfo, Leak-Lookup, HaveIBeenPwned, Emailrep, Dehashed and hunterio",
        nargs="+",
    )
    parser.add_argument(
        "-o", "--output", dest="output_file", help="File to write CSV output"
    )
    parser.add_argument(
        "-bc",
        "--breachcomp",
        dest="bc_path",
        help="Path to the breachcompilation torrent folder. Uses the query.sh script included in the torrent",
    )
    parser.add_argument(
        "-sk",
        "--skip-defaults",
        dest="skip_defaults",
        help="Skips HaveIBeenPwned and HunterIO check. Ideal for local scans",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "-k",
        "--apikey",
        dest="cli_apikeys",
        help='Pass config options. Supported format: "K=V,K=V"',
        nargs="+",
    )
    parser.add_argument(
        "-lb",
        "--local-breach",
        dest="local_breach_src",
        help="Local cleartext breaches to scan for targets. Uses multiprocesses, one separate process per file, on separate worker pool by arguments. Supports file or folder as input, and filepath globing",
        nargs="+",
    )
    parser.add_argument(
        "-gz",
        "--gzip",
        dest="local_gzip_src",
        help="Local tar.gz (gzip) compressed breaches to scans for targets. Uses multiprocesses, one separate process per file. Supports file or folder as input, and filepath globing. Looks for 'gz' in filename",
        nargs="+",
    )
    parser.add_argument(
        "-sf",
        "--single-file",
        dest="single_file",
        help="If breach contains big cleartext or tar.gz files, set this flag to view the progress bar. Disables concurrent file searching for stability",
        action="store_true",
        default=False,
    ),
    parser.add_argument(
        "-ch",
        "--chase",
        dest="chase_limit",
        help="Add related emails from hunter.io to ongoing target list. Define number of emails per target to chase. Requires hunter.io private API key if used without power-chase",
        type=int,
        nargs="?",
    ),
    parser.add_argument(
        "--power-chase",
        dest="power_chase",
        help="Add related emails from ALL API services to ongoing target list. Use with --chase",
        action="store_true",
        default=False,
    ),
    parser.add_argument(
        "--hide",
        dest="hide",
        help="Only shows the first 4 characters of found passwords to output. Ideal for demonstrations",
        action="store_true",
        default=False,
    ),
    parser.add_argument(
        "--debug",
        dest="debug",
        help="Print request debug information",
        action="store_true",
        default=False,
    ),
    parser.add_argument(
        "--gen-config",
        "-g",
        dest="gen_config",
        help="Generates a configuration file template in the current working directory & exits. Will overwrite existing h8mail_config.ini file",
        action="store_true",
        default=False,
    ),

    return parser.parse_args(args)


def main():

    print_banner("warn")
    print_banner("version")
    print_banner()
    check_latest_version()
    user_args = parse_args(sys.argv[1:])
    if user_args.gen_config:
        gen_config_file()
        exit(0)
    h8mail(user_args)



# -*- coding: utf-8 -*-
from .colors import colors as c
from time import sleep

def print_results(results, hide=False):

    for t in results:
        print()
        c.print_res_header(t.target)
        for i in range(len(t.data)):
            # sleep(0.001)
            if len(t.data) == 1:
                print()
                c.info_news("No results founds")
                continue
            if len(t.data[i]) >= 2:  # Contains header + body data
                if hide:
                    if "PASS" in t.data[i][0]:
                        c.print_result(
                            t.target, t.data[i][1][:4] + "********", t.data[i][0]
                        )
                        continue
                    if "LOCAL" in t.data[i][0]:
                        c.print_result(
                            t.target, t.data[i][1][:-5] + "********", t.data[i][0]
                        )
                        continue
                if "HIBP" in t.data[i][0]:
                    c.print_result(t.target, t.data[i][1], t.data[i][0])
                if "HUNTER_PUB" in t.data[i][0]:
                    c.print_result(
                        t.target, str(t.data[i][1]) + " RELATED EMAILS", "HUNTER_PUB"
                    )
                if "HUNTER_RELATED" in t.data[i][0]:
                    c.print_result(t.target, t.data[i][1], "HUNTER_RELATED")
                if "EMAILREP" in t.data[i][0]:
                    c.print_result(
                        t.target, str(t.data[i][1]), t.data[i][0]
                    )
                if "SNUS" in t.data[i][0]:
                    c.print_result(t.target, t.data[i][1], t.data[i][0])
                if "LOCAL" in t.data[i][0]:
                    c.print_result(t.target, t.data[i][1], t.data[i][0])
                if "BC_PASS" in t.data[i][0]:
                    c.print_result(t.target, t.data[i][1], t.data[i][0])
                if "LEAKLOOKUP" in t.data[i][0]:
                    c.print_result(t.target, t.data[i][1], t.data[i][0])
                if "LKLP" in t.data[i][0]:
                    c.print_result(t.target, t.data[i][1], t.data[i][0])
                if "WLI" in t.data[i][0]:
                    c.print_result(t.target, t.data[i][1], t.data[i][0])
                if "SCYLLA" in t.data[i][0]:
                    c.print_result(t.target, t.data[i][1], t.data[i][0])
                if "DHASHD" in t.data[i][0]:
                    c.print_result(t.target, t.data[i][1], t.data[i][0])
                if "INTELX" in t.data[i][0]:
                    c.print_result(t.target, t.data[i][1], t.data[i][0])
    



# -*- coding: utf-8 -*-
from .colors import colors as c
import os
import re
import stat
from string import printable


def breachcomp_check(targets, breachcomp_path):
    # https://gist.github.com/scottlinux/9a3b11257ac575e4f71de811322ce6b3
    try:
        import subprocess

        query_bin = os.path.join(breachcomp_path, "query.sh")
        st = os.stat(query_bin)
        os.chmod(query_bin, st.st_mode | stat.S_IEXEC)
        for t in targets:
            c.info_news(f"Looking up {t.target} in BreachCompilation")
            procfd = subprocess.run([query_bin, t.target], stdout=subprocess.PIPE)
            try:
                output = procfd.stdout.decode("cp437")
            except Exception as e:
                c.bad_news(f"Could not decode bytes for {t.target} results")
                output = procfd.stdout
                # print(output[:85], "[...]")
                print(output)
                continue
            if len(output) != 0:
                split_output = output.split("\n")
                for line in split_output:
                    if ":" in line:
                        t.pwned += 1
                        t.data.append(("BC_PASS", re.split("[;:]",line)[-1]))
                        c.good_news(
                            f"Found BreachedCompilation entry {line}"
                        )
        return targets
    except Exception as e:
        c.bad_news("Breach compilation")
        print(e)
        return targets



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
from .colors import colors as c
import configparser
import csv
import os
import glob
from .version import __version__
import requests
import json


def find_files(to_parse, pattern=""):
    """
    Returns list of files from t_parse filepath.
    Supports using globing (*) in filepaths.
    Can check for patterns such as 'gz'.
    """
    allfiles = []
    if "*" in to_parse:
        glob_result = glob.glob(to_parse)
        for g in glob_result:
            allfiles.append(g)
            c.info_news(f"Using file {g}")
    if os.path.isfile(to_parse):
        if pattern in to_parse:
            c.info_news(f"Using file {to_parse}")
            allfiles.append(to_parse)
    elif os.path.isdir(to_parse):
        for root, _, filenames in os.walk(to_parse):
            for filename in filenames:
                if pattern in filename:
                    c.info_news("Using file {}".format(os.path.join(root, filename)))
                    allfiles.append(os.path.join(root, filename))
    return allfiles


def print_banner(b_type="intro"):
    if "intro" in b_type:
        banner = """
	._____. ._____.     ;____________;
	| ._. | | ._. |     ;   h8mail   ;
	| !_| |_|_|_! |     ;------------;
	!___| |_______!  Heartfelt Email OSINT
	.___|_|_| |___.    Use responsibly
	| ._____| |_. | ;____________________;
	| !_! | | !_! | ; github.com/khast3x ;
	!_____! !_____! ;--------------------;
	"""
        # print(c.bold, c.fg.pink, banner, c.reset)
        banner_tab = banner.splitlines()
        code = 33
        keep = True
        for b in banner_tab:
            clr = "\u001b[38;5;" + str(code) + "m "
            print(c.bold + clr + b + c.reset)
            if keep:
                code += 36
                keep = False
            else:
                keep = True
    elif "warn" in b_type:
        print(
            c.fg.lightgrey,
            "\t\t  Official h8mail posts: \n\t\t  https://khast3x.club/tags/h8mail/\n\n",
            c.reset,
        )
    elif "version" in b_type:
        print(
            "\t",
            c.fg.pink,
            "Version " + __version__ + ' - "ROCKSMASSON.4" ',
            c.reset,
        )


def fetch_emails(target, user_args):
    """
    Returns a list of emails found in 'target'.
    Can be loosy to skip email pattern search.
    """
    if user_args.loose or user_args.user_query is not None:
        t = target.split(" ")
        # print(t)
        return t
    e = re.findall(r"[\w\.-]+@[\w\.-]+", target)
    if e:
        # print(", ".join(e), c.reset)
        return e
    return None


def get_emails_from_file(targets_file, user_args):
    """
    For each line in file, check for emails using fetch_emails().
    Returns list of emails.
    """
    email_obj_list = []
    try:
        target_fd = open(targets_file).readlines()
        c.info_news("Parsing emails from" + targets_file)
        for line in target_fd:
            e = fetch_emails(line.strip(), user_args)
            if e is None:
                continue
            else:
                email_obj_list.extend(e)
        return email_obj_list
    except Exception as ex:
        c.bad_news("Problems occurred while trying to get emails from file")
        print(ex)


def get_config_from_file(user_args):
    """
    Read config in file. If keys are passed using CLI, add them to the configparser object.
    Returns a configparser object already set to "DEFAULT" section.
    """

    try:
        config = configparser.ConfigParser()
        # Config file
        if user_args.config_file:
            for counter, config_file in enumerate(user_args.config_file):
                config_file = user_args.config_file[counter]
                config.read(config_file)
        # Use -k option
        if user_args.cli_apikeys:
            if config.has_section("h8mail") is False:
                config.add_section("h8mail")
            for counter, user_key in enumerate(user_args.cli_apikeys):
                user_cli_keys = user_args.cli_apikeys[counter].split(",")
                for user_key in user_cli_keys:
                    if user_key and "=" in user_key:
                        config.set(
                            "h8mail",
                            user_key.split("=", maxsplit=1)[0].strip(),
                            user_key.split("=", maxsplit=1)[1].strip(),
                        )
            for k in config["h8mail"]:
                if len((config["h8mail"][k])) != 0:
                    c.good_news(f"Found {k} configuration key")
        return config["h8mail"]
    except Exception as ex:
        c.bad_news("Problems occurred while trying to get configuration file")
        print(ex)


def save_results_csv(dest_csv, target_obj_list):
    """
    Outputs CSV from target object list.
    Dumps the target.data object variable into CSV file.
    """
    with open(dest_csv, "w", newline="") as csvfile:
        try:
            writer = csv.writer(csvfile)

            writer.writerow(["Target", "Type", "Data"])
            c.good_news("Writing to CSV")
            for t in target_obj_list:
                for i in range(len(t.data)):
                    if len(t.data[i]) == 2:  # Contains data header + body
                        writer.writerow([t.target, t.data[i][0], t.data[i][1]])
        except Exception as ex:
            c.bad_news("Error writing to csv")
            print(ex)


def check_latest_version():
    """
    Fetches local version and compares it to github api tag version
    """
    try:
        response = requests.request(
            url="https://api.github.com/repos/khast3x/h8mail/releases/latest", method="GET"
        )
        data = response.json()
        latest = data["tag_name"]
        if __version__ == data["tag_name"]:
            c.good_news("h8mail is up to date")
        else:
            c.bad_news(
                "Not running latest h8mail version. [Current: {current} | Latest: {latest}]".format(
                    current=__version__, latest=latest
                )
            )
    except Exception:
        c.bad_news("Could not check for updates. Is Github blocking requests?")
def check_scylla_online():
    """
    Checks if scylla.sh is online
    """
    # Supress SSL Warning on UI
    # https://github.com/khast3x/h8mail/issues/64
    try:
        re = requests.head(
            url="https://scylla.sh", verify=False, auth=requests.auth.HTTPBasicAuth("sammy", "BasicPassword!"), timeout=10,
        )
        if re.status_code == 200:
            c.good_news("scylla.sh is up")
            return True
        else:
            c.info_news("scylla.sh is down, skipping")
        return False
    except Exception:
        c.info_news("scylla.sh is down, skipping")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from .intelx import intelx as i
from time import sleep
from .colors import colors as c


def intelx_getsearch(target, intelx, maxfile):

    c.info_news("[" + target + "]>[intelx.io] Starting search")
    cap = intelx.GET_CAPABILITIES()
    # print(cap["buckets"])
    # import json
    # print(json.dumps(cap, indent=4))
    c.info_news(
        "["
        + target
        + "]>[intelx.io] Search credits remaining : {creds}".format(
            creds=cap["paths"]["/intelligent/search"]["Credit"]
        )
    )
    c.info_news("[" + target + "]>[intelx.io] Search in progress (max results : "+ str(maxfile) + ")")
    search = intelx.search(
        target,
        buckets=["leaks.public", "leaks.private", "pastes", "darknet.tor"],
        # buckets=[],
        maxresults=maxfile,
        media=24,
    )
    c.good_news("[" + target + "]>[intelx.io] Search returned the following files :")
    for record in search["records"]:
        c.good_news("Name: " + record["name"])
        c.good_news("Bucket: " + record["bucket"])
        c.good_news(
            "Size: " + "{:,.0f}".format(record["size"] / float(1 << 20)) + " MB"
        )
        c.info_news("Storage ID: " + record["storageid"])
        print("----------")
    return search



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re
import requests

from .colors import colors as c
from .version import __version__
from .helpers import get_emails_from_file
from .helpers import fetch_emails

def fetch_urls(target):
    """
    Returns a list of URLs found in 'target'.
    """
    # https://www.geeksforgeeks.org/python-check-url-string/
    e = re.findall(
        "http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\), ]|(?:%[0-9a-fA-F][0-9a-fA-F]))+",
        target,
    )
    if e:
        # print(", ".join(e), c.reset)
        return e
    return None


def get_urls_from_file(targets_file):
    """
    For each line in file, check for URLs using fetch_urls().
    Returns list of URLs.
    """
    email_obj_list = []
    c.info_news("Parsing urls from \t" + targets_file)
    try:
        target_fd = open(targets_file).readlines()
        for line in target_fd:
            e = fetch_urls(line)
            if e is None:
                continue
            else:
                email_obj_list.extend(e)
        return email_obj_list
    except Exception as ex:
        c.bad_news("Problems occurred while trying to get URLs from file")
        print(ex)



def worker_url(url):
    """
    Fetches the URL without the h8mail UA
    """
    paramsUA = {"User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/79.0.3945.88 Safari/537.36"}
    try:
        c.info_news("Worker fetching " + url)
        r = requests.get(url, params = paramsUA, allow_redirects=False)
        c.info_news("Worker done fetch url")
        print(f"Status code: {r.status_code}")
    
        e = re.findall(r"[\w\.-]+@[\w\.-]+", r.text)
        print(e)
        if e:
            print(", ".join(e), c.reset)
            return e
        return None
    except Exception as ex:
        c.bad_news("URL fetch worker error:")
        print(ex)


def target_urls(user_args):
    """
    For each user input with --url, check if its a file.
    If yes open and parse each line with regexp, else parse the input with regexp directly.
    Parse html pages from URLs for email patterns.
    Returns list of email targets
    """
    try:
        c.info_news("Starting URL fetch")
        urls = []
        emails = []
        for arg in user_args.user_urls:
            if os.path.isfile(arg):
                e = get_urls_from_file(arg)
            else:
                e = fetch_urls(arg)
            if e is None:
                continue
            else:
                urls.extend(e)
        
        for url in urls:
            e = worker_url(url)
            # e = get_emails_from_file(tmpfile, user_args)
            if e is None:
                continue
            else:
                emails.extend(e)

        return emails
    except Exception as ex:
        c.bad_news("URL fetch error:")
        print(ex)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import signal

from multiprocessing import Pool
from itertools import takewhile, repeat
from .classes import local_breach_target
from .colors import colors as c


def local_to_targets(targets, local_results, user_args):
    """
    Appends data from local_breach_target objects using existing list of targets.
    Finds corresponding email in dest object list, and adds data to the t.data object variable.
    Full output line is stored in t.data[1] and original found data in t.data[2]

    """
    for t in targets:
        for l in local_results:
            if l.target == t.target:
                t.data.append(
                    (
                        "LOCALSEARCH",
                        f"[{os.path.basename(l.filepath)}] Line {l.line}: {l.content}".strip(),
                        l.content.strip(),
                    )
                )
                t.pwned += 1
                if user_args.debug:
                    c.debug_news(f"DEBUG: Found following content matching {t.target.target}")
                    l.target.dump()
    return targets


def raw_in_count(filename):
    """
    StackOverflow trick to rapidly count lines in big files.
    Returns total line number.
    """
    c.info_news("Identifying total line number...")
    f = open(filename, "rb")
    bufgen = takewhile(lambda x: x, (f.raw.read(1024 * 1024) for _ in repeat(None)))
    return sum(buf.count(b"\n") for buf in bufgen)


def worker(filepath, target_list):
    """
    Searches for every email from target_list in every line of filepath.
    Attempts to decode line using cp437. If it fails, catch and use raw data
    """
    try:
        with open(filepath, "rb") as fp:
            found_list = []
            size = os.stat(filepath).st_size
            c.info_news(
                "Worker [{PID}] is searching for targets in {filepath} ({size:,.0f} MB)".format(
                    PID=os.getpid(), filepath=filepath, size=size / float(1 << 20))
                )
            for cnt, line in enumerate(fp):
                for t in target_list:
                    if t in str(line, "cp437"):
                        try:
                            decoded = str(line, "cp437")
                            found_list.append(
                                local_breach_target(t, filepath, cnt, decoded)
                            )
                            c.good_news(
                                f"Found occurrence [{filepath}] Line {cnt}: {decoded}"
                            )
                        except Exception as e:
                            c.bad_news(
                                f"Got a decoding error line {cnt} - file: {filepath}"
                            )
                            c.good_news(
                                f"Found occurrence [{filepath}] Line {cnt}: {line}"
                            )
                            found_list.append(
                                local_breach_target(t, filepath, cnt, str(line))
                            )
        return found_list
    except Exception as e:
        c.bad_news("Something went wrong with worker")
        print(e)


def local_search(files_to_parse, target_list):
    original_sigint_handler = signal.signal(signal.SIGINT, signal.SIG_IGN)
    pool = Pool()
    found_list = []
    signal.signal(signal.SIGINT, original_sigint_handler)
    try:
        async_results = [
            pool.apply_async(worker, args=(f, target_list))
            for i, f in enumerate(files_to_parse)
        ]
        for r in async_results:
            if r.get() is not None:
                found_list.extend(r.get(60))
    except KeyboardInterrupt:
        c.bad_news("Caught KeyboardInterrupt, terminating workers")
        pool.terminate()
    else:
        c.info_news("Terminating worker pool")
        pool.close()
    pool.join()
    return found_list


def progress(count, total, status=""):
    bar_len = 60
    filled_len = int(round(bar_len * count / float(total)))

    percents = round(100.0 * count / float(total), 1)
    bar = "=" * filled_len + "-" * (bar_len - filled_len)

    sys.stdout.write("[%s] %s%s ...%s\r" % (bar, percents, "%", status))
    sys.stdout.write("\033[K")


sys.stdout.flush()


def local_search_single(files_to_parse, target_list):
    found_list = []
    for file_to_parse in files_to_parse:
        with open(file_to_parse, "rb") as fp:
            size = os.stat(file_to_parse).st_size
            lines_no = raw_in_count(file_to_parse)
            c.info_news(
                f"Searching for targets in {file_to_parse} ({size} bytes, {lines_no} lines)"
            )
            for cnt, line in enumerate(fp):
                lines_left = lines_no - cnt
                progress(
                    cnt, lines_no, f"{cnt} lines checked - {lines_left} lines left"
                )
                for t in target_list:
                    if t in str(line):
                        try:
                            decoded = str(line, "cp437")
                            found_list.append(
                                local_breach_target(t, file_to_parse, cnt, decoded)
                            )
                            c.good_news(
                                f"Found occurrence [{file_to_parse}] Line {cnt}: {decoded}"
                            )
                        except:
                            c.bad_news(
                                f"Got a decoding error line {cnt} - file: {file_to_parse}"
                            )
                            c.good_news(
                                f"Found occurrence [{file_to_parse}] Line {cnt}: {line}"
                            )
                            found_list.append(
                                local_breach_target(t, file_to_parse, cnt, str(line))
                            )
    return found_list



#!/usr/bin/env python3

import requests
import time
import json
import sys
import re

class intelx:

	# API_ROOT = 'https://public.intelx.io'
	API_ROOT = ''
	API_KEY  = ''
	USER_AGENT = ''

	# If an API key isn't supplied, it will use the free API key (limited functionality)
	def __init__(self, key="01a61412-7629-4288-b18a-b287266f2798", ua='IX-Python/0.5'):
		"""
		Initialize API by setting the API key.
		"""
		if key == "01a61412-7629-4288-b18a-b287266f2798" or key == "ac572eea-3902-4e9a-972d-f5996d76174c":
			self.API_ROOT 	= "https://public.intelx.io"
		else:
			self.API_ROOT 	= "https://2.intelx.io"

		self.API_KEY = key
		self.USER_AGENT = ua

	def get_error(self, code):
		"""
		Get error string by respective HTTP response code.
		"""
		if code == 200:
			return "200 | Success"
		if code == 204:
			return "204 | No Content"
		if code == 400:
			return "400 | Bad Request"
		if code == 401:
			return "401 | Unauthorized"
		if code == 402:
			return "402 | Payment required."
		if code == 404:
			return "404 | Not Found"

	def cleanup_treeview(self, treeview):
		"""
		Cleans up treeview output from the API.
		"""
		lines = []
		for line in treeview.split("\r\n"):
			if '<a href' not in line:
				lines.append(line)
		return lines
	
	def GET_CAPABILITIES(self):
		"""
		Return a JSON object with the current user's API capabilities
		"""
		h = {'x-key' : self.API_KEY, 'User-Agent': self.USER_AGENT}
		r = requests.get(f"{self.API_ROOT}/authenticate/info", headers=h)
		return r.json()

	def FILE_PREVIEW(self, ctype, mediatype, format, sid, bucket='', e=0, lines=8):
		"""
		Show a preview of a file's contents based on its storageid (sid).
		format option:
		- 0: Text
		- 1: Picture
		"""
		r = requests.get(f"{self.API_ROOT}/file/preview?c={ctype}&m={mediatype}&f={format}&sid={sid}&b={bucket}&e={e}&l={lines}&k={self.API_KEY}")
		return r.text

	def FILE_VIEW(self, ctype, mediatype, sid, bucket='',escape=0):
		"""
		Show a file's contents based on its storageid (sid), convert to text where necessary.

		format option:
		- 0: textview of content
		- 1: hex view of content
		- 2: auto detect hex view or text view
		- 3: picture view
		- 4: not supported
		- 5: html inline view (sanitized)
		- 6: text view of pdf
		- 7: text view of html
		- 8: text view of word file
		"""
		format = 0
		if(mediatype==23 or mediatype==9):	# HTML
			format = 7
		elif(mediatype==15):				# PDF
			format = 6
		elif(mediatype==16):				# Word
			format = 8
		elif(mediatype==18):				# PowerPoint
			format = 10
		elif(mediatype==25):				# Ebook
			format = 11
		elif(mediatype==17):				# Excel
			format = 9
		elif(ctype==1):						# Text
			format = 0
		else:
			format = 1
		r = requests.get(f"{self.API_ROOT}/file/view?f={format}&storageid={sid}&bucket={bucket}&escape={escape}&k={self.API_KEY}")
		return r.text
	
	def FILE_READ(self, id, type=0, bucket="", filename=""):
		"""
		Read a file's raw contents. Use this for direct data download.

		id option:
		- Specifies the item's system ID to read.

		type option:
		- Specifies content disposition or not.
		- 0: No content disposition. Returns raw binary file.
		- 1: Content disposition. May fix line endings to CR LF for text files.

		bucket option:
		- Optional bucket value.

		name option:
		- Specify the name to save the file as (e.g document.pdf).
		"""
		h = {'x-key' : self.API_KEY, 'User-Agent': self.USER_AGENT}
		r = requests.get(f"{self.API_ROOT}/file/read?type={type}&systemid={id}&bucket={bucket}", headers=h, stream=True)
		with open(f"{filename}", "wb") as f:
			f.write(r.content)
			f.close()
		return True

	def FILE_TREE_VIEW(self, sid):
		"""
		Show a treeview of an item that has multiple files/folders
		"""
		try:
			r = requests.get(f"{self.API_ROOT}/file/view?f=12&storageid={sid}&k={self.API_KEY}", timeout=5)
			if "Could not generate" in r.text:
				return False
			return r.text
		except:
			return False

	def INTEL_SEARCH(self, term, maxresults=100, buckets=[], timeout=5, datefrom="", dateto="", sort=4, media=0, terminate=[]):
		"""
		Initialize an intelligent search and return the ID of the task/search for further processing.

		maxresults option:
		- Tells how many results to query maximum per bucket.
		- Example: maxresults=100

		buckets option:
		- Specify the buckets to search
		- Example: buckets=[]
		- Example: buckets=['pastes', 'darknet.i2p']

		timeout option:
		- Set a timeout value for the search.
		- Example: timeout=5

		datefrom option:
		- Set a starting date to begin the search from.
		- Example: 2020-01-01 00:00:00
		- Example: 2020-01-01 12:00:00

		dateto option:
		- Set an ending date to finish the search from.
		- Example: 2020-02-02 23:59:59
		- Example: 2020-02-02 00:00:00

		sort option:
		- Define the way to sort search results.
		- 0: No sorting.
		- 1: X-Score ASC. Least relevant items first.
		- 2: X-Score DESC. Most relevant items first.
		- 3: Date ASC. Oldest items first.
		- 4: Date DESC. Newest items first.

		media option:
		- Define the type of media to search for.
		- 0: Not set. (All media types)
		- 1: Paste document
		- 2: Paste User
		- 3: Forum
		- 4: Forum Board
		- 5: Forum Thread
		- 6: Forum Post
		- 7: Forum User
		- 8: Screenshot of a Website
		- 9: HTML copy of a website.
		- 10: Invalid, do not use.
		- 11: Invalid, do not use.
		- 12: Invalid, do not use.
		- 13: Tweet
		- 14: URL, high-level item having HTML copies as linked sub-items
		- 15: PDF document
		- 16: Word document
		- 17: Excel document
		- 18: Powerpoint document
		- 19: Picture
		- 20: Audio file
		- 21: Video file
		- 22: Container files including ZIP, RAR, TAR and others
		- 23: HTML file
		- 24: Text file

		The term must be a strong selector. These selector types are currently supported:
		- Email address
		- Domain, including wildcards like *.example.com
		- URL
		- IPv4 & IPv6
		- CIDRv4 & CIDRv6
		- Phone Number
		- Bitcoin Address
		- MAC Address
		- IPFS Hash
		- UUID
		- Storage ID
		- System ID
		- Simhash
		- Credit card number
		- IBAN

		Soft selectors (generic terms) are not supported!
		"""
		h = {'x-key' : self.API_KEY, 'User-Agent': self.USER_AGENT}
		p = {
			"term": term,
			"buckets": buckets,
			"lookuplevel": 0,
			"maxresults": maxresults,
			"timeout": timeout,
			"datefrom": datefrom,
			"dateto": dateto,
			"sort": sort,
			"media": media,
			"terminate": terminate
		}
		r = requests.post(self.API_ROOT + '/intelligent/search', headers=h, json=p)
		if r.status_code == 200:
			return r.json()['id']
		else:
			return r.status_code

	def INTEL_SEARCH_RESULT(self, id, limit):
		"""
		Return results from an initialized search based on its ID

		status (results status):
		- 0: Sucess with results.
		- 1: No more results available.
		- 2: Search ID not found.
		- 3: No results yet, but keep trying.

		type (low-level data type of result):
		- 0: Binary/unspecified
		- 1: Plain text
		- 2: Picture
		- 3: Video
		- 4: Audio
		- 5: Document file
		- 6: Executable file
		- 7: Container
		- 1001: User
		- 1002: Leak
		- 1004: URL
		- 1005: Forum

		media (high-level data type):
		- 0: Invalid
		- 1: Paste document
		- 2: Paste user
		- 3: Forum
		- 4: Forum Board
		- 5: Forum Thread
		- 6: Forum Post
		- 7: Forum User
		- 8: Screenshot of a website
		- 9: HTML copy of a website
		- 10: Invalid, do not use.
		- 11: Invalid, do not use.
		- 12: Invalid, do not use.
		- 13: Tweet
		- 14: URL, high-level item having HTML copies as linked sub-items.
		- 15: PDF Document
		- 16: Word document
		- 17: Excel document
		- 18: Powerpoint document
		- 19: Picture
		- 20: Audio file
		- 21: Video file
		- 22: Container files
		- 23: HTML file
		- 24: Text file

		added:
		- When the item was indexed.

		date:
		- The date of the original record if available.

		name:
		- Name of title

		description:
		- Typically not used

		xscore:
		- Relevancy score, between 0-100

		simhash:
		- Similarity hash, depending on the content type

		bucket:
		- The bucket in which the result was found.

		keyvalues:
		- Not used

		tags:
		- Additional information on the item

		relations:
		- Identifiers of related items

		"""
		h = {'x-key' : self.API_KEY, 'User-Agent': self.USER_AGENT}
		r = requests.get(self.API_ROOT + f'/intelligent/search/result?id={id}&limit={limit}', headers=h)
		if(r.status_code == 200):
			return r.json()
		else:
			return r.status_code

	def INTEL_TERMINATE_SEARCH(self, uuid):
		"""
		Terminate a previously initialized search based on its UUID.
		"""
		h = {'x-key' : self.API_KEY, 'User-Agent': self.USER_AGENT}
		r = requests.get(self.API_ROOT + f'/intelligent/search/terminate?id={uuid}', headers=h)
		if(r.status_code == 200):
			return True
		else:
			return r.status_code
	
	def PHONEBOOK_SEARCH(self, term, maxresults=100, buckets=[], timeout=5, datefrom="", dateto="", sort=4, media=0, terminate=[], target=0):
		"""
		Initialize a phonebook search and return the ID of the task/search for further processing
		"""
		h = {'x-key' : self.API_KEY, 'User-Agent': self.USER_AGENT}
		p = {
			"term": term,
			"buckets": buckets,
			"lookuplevel": 0,
			"maxresults": maxresults,
			"timeout": timeout,
			"datefrom": datefrom,
			"dateto": dateto,
			"sort": sort,
			"media": media,
			"terminate": terminate,
			"target": target
		}
		r = requests.post(self.API_ROOT + '/phonebook/search', headers=h, json=p)
		if r.status_code == 200:
			return r.json()['id']
		else:
			return r.status_code

	def PHONEBOOK_SEARCH_RESULT(self, id, limit=1000, offset=-1):
		"""
		Fetch results from a phonebook search based on ID.
		offset:
		- Do not use. If omitted (default), each call will get the next available results.
		___________________________________________
		RETURN VALUES
		status (results status):
		- 0: Sucess with results.
		- 1: No more results available.
		- 2: Search ID not found.
		- 3: No results yet, but keep trying.
		"""
		h = {'x-key' : self.API_KEY, 'User-Agent': self.USER_AGENT}
		r = requests.get(self.API_ROOT + f'/phonebook/search/result?id={id}&limit={limit}&offset={offset}', headers=h)
		if(r.status_code == 200):
			return r.json()
		else:
			return r.status_code

	def query_results(self, id, limit):
		"""
		Query the results from an intelligent search.
		Meant for usage within loops.
		"""
		results = self.INTEL_SEARCH_RESULT(id, limit)
		return results

	def query_pb_results(self, id, limit):
		"""
		Query the results fom a phonebook search.
		Meant for usage within loops.
		"""
		results = self.PHONEBOOK_SEARCH_RESULT(id, limit)
		return results

	def search(self, term, maxresults=100, buckets=[], timeout=5, datefrom="", dateto="", sort=4, media=0, terminate=[]):
		"""
		Conduct a simple search based on a search term.
		Other arguments have default values set, however they can be overridden to complete an advanced search.
		
		maxresults option:
		- Tells how many results to query maximum per bucket.
		- Example: maxresults=100

		buckets option:
		- Specify the buckets to search
		- Example: buckets=[]
		- Example: buckets=['pastes', 'darknet.i2p']

		timeout option:
		- Set a timeout value for the search.
		- Example: timeout=5

		datefrom option:
		- Set a starting date to begin the search from.
		- Example: 2020-01-01 00:00:00
		- Example: 2020-01-01 12:00:00

		dateto option:
		- Set an ending date to finish the search from.
		- Example: 2020-02-02 23:59:59
		- Example: 2020-02-02 00:00:00

		sort option:
		- Define the way to sort search results.
		- 0: No sorting.
		- 1: X-Score ASC. Least relevant items first.
		- 2: X-Score DESC. Most relevant items first.
		- 3: Date ASC. Oldest items first.
		- 4: Date DESC. Newest items first.

		media option:
		- Define the type of media to search for.
		- 0: Not set. (All media types)
		- 1: Paste document
		- 2: Paste User
		- 3: Forum
		- 4: Forum Board
		- 5: Forum Thread
		- 6: Forum Post
		- 7: Forum User
		- 8: Screenshot of a Website
		- 9: HTML copy of a website.
		- 10: Invalid, do not use.
		- 11: Invalid, do not use.
		- 12: Invalid, do not use.
		- 13: Tweet
		- 14: URL, high-level item having HTML copies as linked sub-items
		- 15: PDF document
		- 16: Word document
		- 17: Excel document
		- 18: Powerpoint document
		- 19: Picture
		- 20: Audio file
		- 21: Video file
		- 22: Container files including ZIP, RAR, TAR and others
		- 23: HTML file
		- 24: Text file

		The term must be a strong selector. These selector types are currently supported:
		- Email address
		- Domain, including wildcards like *.example.com
		- URL
		- IPv4 & IPv6
		- CIDRv4 & CIDRv6
		- Phone Number
		- Bitcoin Address
		- MAC Address
		- IPFS Hash
		- UUID
		- Storage ID
		- System ID
		- Simhash
		- Credit card number
		- IBAN

		Soft selectors (generic terms) are not supported!
		
		"""
		results = []
		done = False
		search_id = self.INTEL_SEARCH(term, maxresults, buckets, timeout, datefrom, dateto, sort, media, terminate)
		if(len(str(search_id)) <= 3):
			 print(f"[!] intelx.INTEL_SEARCH() Received {self.get_error(search_id)}")
			 sys.exit()
		while done == False:
			time.sleep(1) # lets give the backend a chance to aggregate our data
			r = self.query_results(search_id, maxresults)
			for a in r['records']:
				results.append(a)
			maxresults -= len(r['records'])
			if(r['status'] == 1 or r['status'] == 2 or maxresults <= 0):
				if(maxresults <= 0):
					self.INTEL_TERMINATE_SEARCH(search_id)
				done = True
		return {'records': results}

	def phonebooksearch(self, term, maxresults=1000, buckets=[], timeout=5, datefrom="", dateto="", sort=4, media=0, terminate=[], target=0):
		"""
		Conduct a phonebook search based on a search term.
		Other arguments have default values set, however they can be overridden to complete an advanced search.
		"""
		results = []
		done = False
		search_id = self.PHONEBOOK_SEARCH(term, maxresults, buckets, timeout, datefrom, dateto, sort, media, terminate, target)
		if(len(str(search_id)) <= 3):
			 print(f"[!] intelx.PHONEBOOK_SEARCH() Received {self.get_error(search_id)}")
			 sys.exit()
		while done == False:
			time.sleep(1) # lets give the backend a chance to aggregate our data
			r = self.query_pb_results(search_id, maxresults)
			results.append(r)
			maxresults -= len(r['selectors'])
			if(r['status'] == 1 or r['status'] == 2 or maxresults <= 0):
				if(maxresults <= 0):
					self.INTEL_TERMINATE_SEARCH(search_id)
				done = True
		return results

	def stats(self, search):
		stats = {}
		for record in search['records']:
			if record['bucket'] not in stats:
				stats[record['bucket']] = 1
			else:
				stats[record['bucket']] += 1
		return json.dumps(stats)



#!/usr/bin/env python
from .intelx import intelx as i
from time import sleep
from .colors import colors as c
import requests
import json
import socket
import sys
import platform
from .version import __version__


class local_breach_target:
    """
	Class is called when performing local file search.
	This class is meant to store found data, to be later appended to the existing target objects.
    local_to_targets() tranforms to target object
	Used by both cleartext and gzip search
	"""

    def __init__(self, target_data, fp, ln, content):
        self.target = target_data
        self.filepath = fp
        self.line = ln
        self.content = content

    def dump(self):
        print(f"Email: {self.target}")
        print(f"Path: {self.filepath}")
        print(f"Line: {self.line}")
        print(f"Content: {self.content}")
        print()


class target:
    """
	Main class used to create and follow breach data.
	Found data is stored in self.data. Each method increments self.pwned when data is found.
	"""

    def __init__(self, target_data, debug=False):
        self.headers = {
            "User-Agent": "h8mail-v.{h8ver}-OSINT-and-Education-Tool (PythonVersion={pyver}; Platform={platfrm})".format(
                h8ver=__version__,
                pyver=sys.version.split(" ")[0],
                platfrm=platform.platform().split("-")[0],
            )
        }
        self.target = target_data
        self.pwned = 0
        self.data = [()]
        self.debug = debug
        if debug:
            print(
                c.fg.red,
                f"DEBUG: Created target object for {self.target}",
                c.reset,
            )

    def not_exists(self, pattern):
        for d in self.data:
            if len(d) >= 2:
                if d[1] == pattern:
                    return False
        return True

    def make_request(
        self,
        url,
        meth="GET",
        timeout=20,
        redirs=True,
        data=None,
        params=None,
        verify=True,
        auth=None,
    ):
        try:
            response = requests.request(
                url=url,
                headers=self.headers,
                method=meth,
                timeout=timeout,
                allow_redirects=redirs,
                data=data,
                params=params,
                verify=verify,
                auth=auth,
            )
            # response = requests.request(url="http://127.0.0.1:8000", headers=self.headers, method=meth, timeout=timeout, allow_redirects=redirs, data=data, params=params)
            if self.debug:
                c.debug_news("DEBUG: Sent the following---------------------")
                print(self.headers)
                print(url, meth, data, params)
                c.debug_news("DEBUG: Received the following---------------------")
                c.debug_news(response.url)
                c.debug_news("DEBUG: RESPONSE HEADER---------------------")
                print(
                    "\n".join(
                        f"{k}: {v}" for k, v in response.headers.items()
                    )
                )
                c.debug_news("DEBUG: RESPONSE BODY---------------------")
                print(response.content)
                # print(response)
        except Exception as ex:
            c.bad_news("Request could not be made for " + self.target)
            print(url)
            print(ex)
            print(response)
        return response

    # New HIBP API
    def get_hibp3(self, api_key):
        try:
            c.info_news("[" + self.target + "]>[hibp]")
            sleep(1.3)
            url = f"https://haveibeenpwned.com/api/v3/breachedaccount/{self.target}"
            self.headers.update({"hibp-api-key": api_key})
            response = self.make_request(url)
            if response.status_code not in [200, 404]:
                c.bad_news("Could not contact HIBP v3 for " + self.target)
                print(response.status_code)
                return

            if response.status_code == 200:
                data = response.json()
                for d in data:  # Returned type is a dict of Name : Service
                    for _, ser in d.items():
                        self.data.append(("HIBP3", ser))
                        self.pwned += 1

                c.good_news(
                    "Found {num} breaches for {target} using HIBP v3".format(
                        num=len(self.data) - 1, target=self.target
                    )
                )
                self.get_hibp3_pastes()
                self.headers.popitem()
            elif response.status_code == 404:
                c.info_news(
                    f"No breaches found for {self.target} using HIBP v3"
                )

            else:
                c.bad_news(
                    f"HIBP v3: got API response code {response.status_code} for {self.target}"
                )

        except Exception as e:
            c.bad_news("haveibeenpwned v3: " + self.target)
            print(e)

    # New HIBP API
    def get_hibp3_pastes(self):
        try:
            c.info_news("[" + self.target + "]>[hibp-paste]")
            sleep(1.3)
            url = f"https://haveibeenpwned.com/api/v3/pasteaccount/{self.target}"

            response = self.make_request(url)
            if response.status_code not in [200, 404]:
                c.bad_news("Could not contact HIBP PASTE for " + self.target)
                print(response.status_code)
                print(response)
                return

            if response.status_code == 200:

                data = response.json()
                for d in data:  # Returned type is a dict of Name : Service
                    self.pwned += 1
                    if "Pastebin" in d["Source"]:
                        self.data.append(
                            ("HIBP3_PASTE", "https://pastebin.com/" + d["Id"])
                        )
                    else:
                        self.data.append(("HIBP3_PASTE", d["Id"]))

                c.good_news(
                    "Found {num} pastes for {target} using HIBP v3 Pastes".format(
                        num=len(data), target=self.target
                    )
                )

            elif response.status_code == 404:
                c.info_news(
                    f"No pastes found for {self.target} using HIBP v3 PASTE"
                )
            else:
                c.bad_news(
                    f"HIBP v3 PASTE: got API response code {response.status_code} for {self.target}"
                )

        except Exception as ex:
            c.bad_news("HIBP v3 PASTE error: " + self.target)
            print(ex)

    def get_intelx(self, api_keys):
        try:
            intel_files = []
            intelx = i(key=api_keys["intelx_key"], ua="h8mail-v.{h8ver}-OSINT-and-Education-Tool (PythonVersion={pyver}; Platform={platfrm})".format(
                h8ver=__version__,
                pyver=sys.version.split(" ")[0],
                platfrm=platform.platform().split("-")[0],
            ))
            from .intelx_helpers import intelx_getsearch
            from .localsearch import local_search
            from os import remove, fspath

            maxfile = 10
            if api_keys["intelx_maxfile"]:
                maxfile = int(api_keys["intelx_maxfile"])
            search = intelx_getsearch(self.target, intelx, maxfile)
            if self.debug:
                import json

                print(json.dumps(search, indent=4))

            for record in search["records"]:
                filename = record["systemid"].strip() + ".txt"
                intel_files.append(filename)
                if record["media"] != 24:
                    c.info_news(
                        "Skipping {name}, not text ({type})".format(
                            type=record["mediah"], name=record["name"]
                        )
                    )
                    continue
                c.good_news(
                    "["
                    + self.target
                    + "]>[intelx.io] Fetching "
                    + record["name"]
                    + " as file "
                    + filename
                    + " ("
                    + "{:,.0f}".format(record["size"] / float(1 << 20))
                    + " MB)"
                )
                intelx.FILE_READ(record["systemid"], 0, record["bucket"], filename)
                found_list = local_search([filename], [self.target])
                for f in found_list:
                    self.pwned += 1
                    self.data.append(
                        (
                            "INTELX.IO",
                            "{name} | Line: {line} - {content}".format(
                                name=record["name"].strip(),
                                line=f.line,
                                content=" ".join(f.content.split()),
                            ),
                        )
                    )
                # print(contents) # Contains search data
            for file in intel_files:
                if self.debug:
                    c.info_news(
                        "["
                        + self.target
                        + f"]>[intelx.io] [DEBUG] Keeping {file}"
                    )
                else:
                    c.info_news(
                        "["
                        + self.target
                        + f"]>[intelx.io] Removing {file}"
                    )
                    remove(file)

        except Exception as ex:
            c.bad_news("intelx.io error: " + self.target)
            print(ex)

    def get_emailrepio(self, api_key=""):
        try:
            sleep(0.5)
            if len(api_key) != 0:
                self.headers.update({"Key": api_key})
                c.info_news("[" + self.target + "]>[emailrep.io+key]")
            else:
                c.info_news("[" + self.target + "]>[emailrep.io]")
            url = f"https://emailrep.io/{self.target}"
            response = self.make_request(url)
            if response.status_code not in [200, 404, 429]:
                c.bad_news("Could not contact emailrep for " + self.target)
                print(response.status_code)
                print(response)
                return

            if response.status_code == 429:
                c.info_news(
                    "[warning] Is your emailrep key working? Get a free API key here: https://bit.ly/3b1e7Pw"
                )
            elif response.status_code == 404:
                c.info_news(
                    f"No data found for {self.target} using emailrep.io"
                )
            elif response.status_code == 200:
                data = response.json()

                self.data.append(
                    (
                        "EMAILREP_INFO",
                        "Reputation: {rep} | Deliverable: {deli}".format(
                            rep=data["reputation"].capitalize(),
                            deli=data["details"]["deliverable"],
                        ),
                    )
                )

                if data["details"]["credentials_leaked"] is True:
                    self.pwned += int(data["references"])  # or inc num references
                    if data["references"] == 1:
                        self.data.append(
                            (
                                "EMAILREP_LEAKS",
                                f"{data['references']} leaked credential",
                            )
                        )
                    else:
                        self.data.append(
                            (
                                "EMAILREP_LEAKS",
                                "{} leaked credentials".format(data["references"]),
                            )
                        )
                    c.good_news(
                        "Found {num} breaches for {target} using emailrep.io".format(
                            num=data["references"], target=self.target
                        )
                    )
                if len(data["details"]["profiles"]) != 0:
                    for profile in data["details"]["profiles"]:
                        self.data.append(("EMAILREP_SOCIAL", profile.capitalize()))
                c.good_news(
                    "Found {num} social profiles linked to {target} using emailrep.io".format(
                        num=len(data["details"]["profiles"]), target=self.target
                    )
                )
                if "never" in data["details"]["last_seen"]:
                    return
                self.data.append(("EMAILREP_1ST_SN", data["details"]["first_seen"]))
                c.good_news(
                    "{target} was first seen on the {data}".format(
                        data=data["details"]["first_seen"], target=self.target
                    )
                )
                self.data.append(("EMAILREP_LASTSN", data["details"]["last_seen"]))
                c.good_news(
                    "{target} was last seen on the {data}".format(
                        data=data["details"]["last_seen"], target=self.target
                    )
                )
            else:
                c.bad_news(
                    "emailrep.io: got API response code {code} for {target}".format(
                        code=response.status_code, target=self.target
                    )
                )
            if len(api_key) != 0:
                self.headers.popitem()
        except Exception as ex:
            c.bad_news("emailrep.io error: " + self.target)
            print(ex)

    def get_scylla(self, user_query="email"):
        try:
            c.info_news("[" + self.target + "]>[scylla.sh]")
            sleep(0.5)
            self.headers.update({"Accept": "application/json"})
            if user_query == "email":
                uri_scylla = 'Email: "' + self.target + '"'
            elif user_query == "password":
                uri_scylla = 'Password: "' + self.target + '"'
            elif user_query == "username":
                uri_scylla = 'User: "' + self.target + '"'
            elif user_query == "ip":
                uri_scylla = 'IP: "' + self.target + '"'
            elif user_query == "hash":
                uri_scylla = 'Hash: "' + self.target + '"'
            elif user_query == "domain":
                uri_scylla = 'Email: "*@' + self.target + '"'
            url = "https://scylla.sh/search?q={}".format(
                requests.utils.requote_uri(uri_scylla)
            )

            # https://github.com/khast3x/h8mail/issues/64
            response = self.make_request(
                url,
                verify=False,
                # auth=requests.auth.HTTPBasicAuth("sammy", "BasicPassword!"),
            )
            self.headers.popitem()

            if response.status_code not in [200, 404]:
                c.bad_news("Could not contact scylla.sh for " + self.target)
                print(response.status_code)
                print(response)
                return
            data = response.json()
            total = 0
            for d in data:
                for field, k in d["fields"].items():
                    if k is not None:
                        total += 1
            c.good_news(
                "Found {num} entries for {target} using scylla.sh ".format(
                    num=total, target=self.target
                )
            )
            for d in data:
                for field, k in d["fields"].items():
                    if "user" in field and k is not None:
                        self.data.append(("SCYLLA_USERNAME", k))
                        self.pwned += 1
                    elif (
                        "email" in field and k is not None and user_query != "email"
                    ):
                        self.data.append(("SCYLLA_EMAIL", k))
                        self.pwned += 1
                    elif "password" in field and k is not None:
                        self.data.append(("SCYLLA_PASSWORD", k))
                        self.pwned += 1
                    elif "passhash" in field and k is not None:
                        self.data.append(("SCYLLA_HASH", k))
                        self.pwned += 1
                    elif "passsalt" in field and k is not None:
                        self.data.append(("SCYLLA_HASHSALT", k))
                        self.pwned += 1
                    elif "ip" in field and k is not None:
                        self.data.append(("SCYLLA_LASTIP", k))
                        self.pwned += 1
                    elif "domain" in field and k is not None:
                        self.data.append(("SCYLLA_SOURCE", k))
                        self.pwned += 1
        except Exception as ex:
            c.bad_news("scylla.sh error: " + self.target)
            print(ex)

    def get_hunterio_public(self):
        try:
            c.info_news("[" + self.target + "]>[hunter.io public]")
            target_domain = self.target.split("@")[1]
            url = f"https://api.hunter.io/v2/email-count?domain={target_domain}"
            req = self.make_request(url)
            response = req.json()
            if response["data"]["total"] != 0:
                self.data.append(("HUNTER_PUB", response["data"]["total"]))
            c.good_news(
                "Found {num} related emails for {target} using hunter.io (public)".format(
                    num=response["data"]["total"], target=self.target
                )
            )
        except Exception as ex:
            c.bad_news("hunter.io (public API) error: " + self.target)
            print(ex)

    def get_hunterio_private(self, api_key):
        try:
            c.info_news("[" + self.target + "]>[hunter.io private]")
            target_domain = self.target.split("@")[1]
            url = f"https://api.hunter.io/v2/domain-search?domain={target_domain}&api_key={api_key}"
            req = self.make_request(url)
            response = req.json()
            b_counter = 0
            for e in response["data"]["emails"]:
                self.data.append(("HUNTER_RELATED", e["value"]))
                b_counter += 1
                if self.pwned != 0:
                    self.pwned += 1
            c.good_news(
                "Found {num} related emails for {target} using hunter.io (private)".format(
                    num=b_counter, target=self.target
                )
            )
        except Exception as ex:
            c.bad_news(
                f"hunter.io (private API) error for {self.target}:"
            )
            print(ex)

    def get_snusbase(self, api_url, api_key, user_query):
        try:
            if user_query == "ip":
                user_query = "lastip"
            if user_query in ["domain"]:
                c.bad_news(
                    f"Snusbase does not support {user_query} search (yet)"
                )
                return
            c.info_news("[" + self.target + "]>[snusbase]")
            url = api_url
            self.headers.update({"Authorization": api_key})
            payload = {"type": user_query, "term": self.target}
            req = self.make_request(url, meth="POST", data=payload)
            self.headers.popitem()
            response = req.json()
            c.good_news(
                "Found {num} entries for {target} using Snusbase".format(
                    num=len(response["result"]), target=self.target
                )
            )
            for result in response["result"]:
                if result["email"] and self.not_exists(result["email"]):
                    self.data.append(("SNUS_RELATED", result["email"].strip()))
                if result["username"]:
                    self.data.append(("SNUS_USERNAME", result["username"]))
                    self.pwned += 1
                if result["password"]:
                    self.data.append(("SNUS_PASSWORD", result["password"]))
                    self.pwned += 1
                if result["hash"]:
                    if result["salt"]:
                        self.data.append(
                            (
                                "SNUS_HASH_SALT",
                                result["hash"].strip() + " : " + result["salt"].strip(),
                            )
                        )
                        self.pwned += 1
                    else:
                        self.data.append(("SNUS_HASH", result["hash"]))
                        self.pwned += 1
                if result["lastip"]:
                    self.data.append(("SNUS_LASTIP", result["lastip"]))
                    self.pwned += 1
                if result["name"]:
                    self.data.append(("SNUS_NAME", result["name"]))
                    self.pwned += 1
                if result["tablenr"] and self.not_exists(result["tablenr"]):
                    self.data.append(("SNUS_SOURCE", result["tablenr"]))

        except Exception as ex:
            c.bad_news(f"Snusbase error with {self.target}")
            print(ex)

    def get_leaklookup_pub(self, api_key):
        try:
            c.info_news("[" + self.target + "]>[leaklookup public]")
            url = "https://leak-lookup.com/api/search"
            payload = {"key": api_key, "type": "email_address", "query": self.target}
            req = self.make_request(url, meth="POST", data=payload, timeout=20)
            response = req.json()
            if "false" in response["error"] and len(response["message"]) != 0:
                c.good_news(
                    "Found {num} entries for {target} using LeakLookup (public)".format(
                        num=len(response["message"]), target=self.target
                    )
                )
                for result in response["message"]:
                    self.pwned += 1
                    self.data.append(("LEAKLOOKUP_PUB", result))
            if "false" in response["error"] and len(response["message"]) == 0:
                c.info_news(
                    f"No breaches found for {self.target} using Leak-lookup (public)"
                )

        except Exception as ex:
            c.bad_news(
                f"Leak-lookup error with {self.target} (public)"
            )
            print(ex)

    def get_leaklookup_priv(self, api_key, user_query):
        try:
            if user_query == "ip":
                user_query = "ipadress"
            if user_query in ["hash"]:
                c.bad_news(
                    f"Leaklookup does not support {user_query} search (yet)"
                )
                return
            c.info_news("[" + self.target + "]>[leaklookup private]")
            url = "https://leak-lookup.com/api/search"
            payload = {"key": api_key, "type": user_query, "query": self.target}
            req = self.make_request(url, meth="POST", data=payload, timeout=60)
            response = req.json()
            if "false" in response["error"] and len(response["message"]) != 0:
                b_counter = 0
                for db, data in response["message"].items():
                    for d in data:
                        if "username" in d.keys():
                            self.pwned += 1
                            self.data.append(("LKLP_USERNAME", d["username"]))
                        if "email_address" in d.keys() and self.not_exists(
                            d["email_address"]
                        ):
                            self.data.append(
                                ("LKLP_RELATED", d["email_address"].strip())
                            )
                        if "password" in d.keys():
                            self.pwned += 1
                            self.data.append(("LKLP_PASSWORD", d["password"]))
                            b_counter += 1
                        if "hash" in d.keys():
                            self.pwned += 1
                            self.data.append(("LKLP_HASH", d["password"]))
                            b_counter += 1
                        if "ipaddress" in d.keys():
                            self.pwned += 1
                            self.data.append(("LKLP_LASTIP", d["ipaddress"]))
                            for tag in [
                                "address",
                                "address1",
                                "address2",
                                "country",
                                "zip",
                                "zipcode",
                                "postcode",
                                "state",
                            ]:
                                if tag in d.keys():
                                    self.pwned += 1
                                    self.data.append(
                                        ("LKLP_GEO", d[tag] + " (type: " + tag + ")")
                                    )
                            for tag in [
                                "firstname",
                                "middlename",
                                "lastname",
                                "mobile",
                                "number",
                                "userid",
                            ]:
                                if tag in d.keys():
                                    self.pwned += 1
                                    self.data.append(
                                        ("LKLP_ID", d[tag] + " (type: " + tag + ")")
                                    )
                    if self.not_exists(db):
                        self.data.append(("LKLP_SOURCE", db))

                c.good_news(
                    "Found {num} entries for {target} using LeakLookup (private)".format(
                        num=b_counter, target=self.target
                    )
                )

            if "false" in response["error"] and len(response["message"]) == 0:
                c.info_news(
                    f"No breaches found for {self.target} using Leak-lookup (private)"
                )
        except Exception as ex:
            c.bad_news(
                f"Leak-lookup error with {self.target} (private)"
            )
            print(ex)

    def get_weleakinfo_priv(self, api_key, user_query):
        try:
            c.info_news("[" + self.target + "]>[weleakinfo priv]")
            sleep(0.4)
            url = "https://api.weleakinfo.com/v3/search"
            self.headers.update({"Authorization": "Bearer " + api_key})
            self.headers.update({"Content-Type": "application/x-www-form-urlencoded"})

            payload = {"type": user_query, "query": self.target}
            req = self.make_request(url, meth="POST", data=payload, timeout=30)
            self.headers.popitem()
            self.headers.popitem()
            response = req.json()
            if req.status_code == 400:
                c.bad_news(
                    f"Got WLI API response code {req.status_code}: Invalid search type provided"
                )
                return
            elif req.status_code != 200:
                c.bad_news(f"Got WLI API response code {req.status_code} (private)")
                return
            if req.status_code == 200:
                if response["Success"] is False:
                    c.bad_news(
                        f"WeLeakInfo (private) error response {response['Message']}"
                    )
                    return
                c.good_news(
                    "Found {num} entries for {target} using WeLeakInfo (private)".format(
                        num=response["Total"], target=self.target
                    )
                )
                self.data.append(("WLI_TOTAL", response["Total"]))
                if response["Total"] == 0:
                    return
                for result in response["Data"]:
                    if "Username" in result:
                        self.data.append(("WLI_USERNAME", result["Username"]))
                    if "Email" in result and self.not_exists(result["Email"]):
                        self.data.append(("WLI_RELATED", result["Email"].strip()))
                    if "Password" in result:
                        self.data.append(("WLI_PASSWORD", result["Password"]))
                        self.pwned += 1
                    if "Hash" in result:
                        self.data.append(("WLI_HASH", result["Hash"]))
                        self.pwned += 1
                    if "Database" in result and self.not_exists(result["Database"]):
                        self.data.append(("WLI_SOURCE", result["Database"]))
        except Exception as ex:
            c.bad_news(
                f"WeLeakInfo error with {self.target} (private)"
            )
            print(ex)

    def get_weleakinfo_pub(self, api_key):
        try:
            c.info_news("[" + self.target + "]>[weleakinfo public]")
            url = "https://api.weleakinfo.com/v3/public/email/{query}".format(
                query=self.target
            )
            self.headers.update({"Authorization": "Bearer " + api_key})
            req = self.make_request(url, timeout=30)
            self.headers.popitem()
            response = req.json()
            if req.status_code != 200:
                c.bad_news(f"Got WLI API response code {req.status_code} (public)")
                return
            else:
                c.good_news(
                    "Found {num} entries for {target} using WeLeakInfo (public)".format(
                        num=response["Total"], target=self.target
                    )
                )
                if response["Success"] is False:
                    c.bad_news(response["Message"])
                    return
                self.data.append(("WLI_PUB_TOTAL", response["Total"]))
                if response["Total"] == 0:
                    return
                for name, data in response["Data"].items():
                    self.data.append(("WLI_PUB_SRC", name + " (" + str(data) + ")"))
        except Exception as ex:
            c.bad_news(
                f"WeLeakInfo error with {self.target} (public)"
            )
            print(ex)

    def get_dehashed(self, api_email, api_key, user_query):
        try:
            if user_query == "hash":
                user_query == "hashed_password"
            if user_query == "ip":
                user_query == "ip_address"

            c.info_news("[" + self.target + "]>[dehashed]")
            url = "https://api.dehashed.com/search?query="
            if user_query == "domain":
                search_query = "email" + ":" + '"*@' + self.target + '"'
            else:
                search_query = user_query + ":" + '"' + self.target + '"'
            self.headers.update({"Accept": "application/json"})
            req = self.make_request(
                url + search_query, meth="GET", timeout=60, auth=(api_email, api_key)
            )
            if req.status_code == 200:
                response = req.json()
                if response["total"] is not None:
                    c.good_news(
                        "Found {num} entries for {target} using Dehashed.com".format(
                            num=str(response["total"]), target=self.target
                        )
                    )

                for result in response["entries"]:
                    if (
                        "username" in result
                        and result["username"] is not None
                        and len(result["username"].strip()) > 0
                    ):
                        self.data.append(("DHASHD_USERNAME", result["username"]))
                    if (
                        "email" in result
                        and self.not_exists(result["email"])
                        and result["email"] is not None
                        and len(result["email"].strip()) > 0
                    ):
                        self.data.append(("DHASHD_RELATED", result["email"].strip()))
                    if (
                        "password" in result
                        and result["password"] is not None
                        and len(result["password"].strip()) > 0
                    ):
                        self.data.append(("DHASHD_PASSWORD", result["password"]))
                        self.pwned += 1
                    if (
                        "hashed_password" in result
                        and result["hashed_password"] is not None
                        and len(result["hashed_password"].strip()) > 0
                    ):
                        self.data.append(("DHASHD_HASH", result["hashed_password"]))
                        self.pwned += 1
                    for tag in ["name", "vin", "address", "phone"]:
                        if (
                            tag in result
                            and result[tag] is not None
                            and len(result[tag].strip()) > 0
                        ):
                            self.data.append(
                                ("DHASHD_ID", result[tag] + " (type: " + tag + ")")
                            )
                            self.pwned += 1
                    # Documentation and JSON are not synced, using both source keys
                    if "obtained_from" in result and self.not_exists(
                        result["obtained_from"]
                    ):
                        self.data.append(("DHASHD_SOURCE", result["obtained_from"]))
                    if "database_name" in result and self.not_exists(
                        result["database_name"]
                    ):
                        self.data.append(("DHASHD_SOURCE", result["database_name"]))

                if response["balance"] is not None:
                    self.data.append(
                        (
                            "DHASHD_CREDITS",
                            str(response["balance"]) + " DEHASHED CREDITS REMAINING",
                        )
                    )
            else:
                c.bad_news("Dehashed error: status code " + req.status_code)
            self.headers.popitem()
        except Exception as ex:
            c.bad_news(f"Dehashed error with {self.target}")
            print(ex)



import sys
from .utils.run import main

if __name__ == "__main__":
    # Check major and minor python version
    if sys.version_info[0] < 3:
        sys.stdout.write(
            "\n/!\\ h8mail requires Python 3.6+ /!\\\nTry running h8mail with python3 if on older systems\n\neg: python --version\neg: python3 h8mail v --help\n\n"
        )
        sys.exit(1)
    if sys.version_info[1] < 6:
        sys.stdout.write(
            "\n/!\\ h8mail requires Python 3.6+ /!\\\nTry running h8mail with python3 if on older systems\n\neg: python --version\neg: python3 h8mail --help\n\n"
        )
        sys.exit(1)
    main()
    print("Done")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""
from setuptools import setup, find_packages, Extension
from distutils.util import convert_path
import os

here = os.path.abspath(os.path.dirname(__file__))
exec(open(os.path.join(here, "h8mail/utils/version.py")).read())

with open("PyPi.rst") as readme_file:
    readme = readme_file.read()

# with open("HISTORY.rst") as history_file:
#     history = history_file.read()

requirements = ["requests"]

setup_requirements = ["requests"]

test_requirements = ["requests"]

setup(
    author="khast3x",
    author_email="k@khast3x.club",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Environment :: Console",
    ],
    description="Email OSINT and password breach hunting. Use h8mail to find passwords through different breach and reconnaissance services, or using your local data",
    install_requires=requirements,
    license="BSD license",
    # long_description_content_type="text/markdown",
    long_description=readme + "\n\n",
    # long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="h8mail",
    name="h8mail",
    packages=find_packages(),
    entry_points={"console_scripts": ["h8mail = h8mail.__main__:main"]},
    setup_requires=setup_requirements,
    url="https://github.com/khast3x/h8mail",
    version=__version__,
    zip_safe=False,
)



# -*- coding: utf-8 -*-


class colors:
    """Colors class:
    reset all colors with colors.reset
    two subclasses fg for foreground and bg for background.
    use as colors.subclass.colorname.
    i.e. colors.fg.red or colors.bg.green
    also, the generic bold, disable, underline, reverse, strikethrough,
    and invisible work with the main class
    i.e. colors.bold
    """

    reset = "\033[0m"
    bold = "\033[01m"
    disable = "\033[02m"
    underline = "\033[04m"
    reverse = "\033[07m"
    strikethrough = "\033[09m"
    invisible = "\033[08m"

    class fg:
        black = "\033[30m"
        red = "\033[31m"
        green = "\033[32m"
        orange = "\033[33m"
        blue = "\033[34m"
        purple = "\033[35m"
        cyan = "\033[36m"
        lightgrey = "\033[37m"
        darkgrey = "\033[90m"
        lightred = "\033[91m"
        lightgreen = "\033[92m"
        yellow = "\033[93m"
        lightblue = "\033[94m"
        pink = "\033[95m"
        lightcyan = "\033[96m"

    class bg:
        black = "\033[40m"
        red = "\033[41m"
        green = "\033[42m"
        orange = "\033[43m"
        blue = "\033[44m"
        purple = "\033[45m"
        cyan = "\033[46m"
        lightgrey = "\033[47m"

    @staticmethod
    def good_news(news):
        """
        Print a Success
        """
        print(colors.bold + colors.fg.green + "[>] " + colors.reset + news.strip())

    @staticmethod
    def debug_news(news):
        """
        Print a Debug
        """
        print()
        print(colors.bold + colors.fg.lightred + "[@] " + news + colors.reset)

    @staticmethod
    def bad_news(news):
        """
        Print a Failure, error
        """
        print(colors.bold + colors.fg.red + "[!] " + colors.reset + news.strip())

    @staticmethod
    def info_news(news):
        """
        Print an information with grey text
        """
        print(
            colors.bold
            + colors.fg.lightblue
            + "[~] "
            + colors.reset
            + colors.fg.lightgrey
            + news.strip()
            + colors.reset
        )

    @staticmethod
    def question_news(news):
        """
        Print an information with yellow text
        """
        print(
            colors.bold
            + colors.fg.blue
            + "[?] "
            + colors.reset
            + colors.fg.yellow
            + news.strip()
            + colors.reset
        )

    @staticmethod
    def print_result(target, data, source):
        """
        Print Breach results
        """
        if "PASS" in source:
            print(
                "{}{}{:15}{}|{}{:>25.25}{} > {}{}{}{}".format(
                    colors.fg.lightblue,
                    colors.bold,
                    source,
                    colors.fg.lightgrey,
                    colors.fg.pink,
                    target,
                    colors.fg.lightgrey,
                    colors.bold,
                    colors.fg.green,
                    data,
                    colors.reset,
                )
            )
        elif "LOCALSEARCH" in source:
            if len(data) > 140:
                print(
                    "{}{}{:15}{}|{}{:>25.25}{} > {}{}{}{}".format(
                        colors.fg.lightblue,
                        colors.bold,
                        source,
                        colors.fg.lightgrey,
                        colors.fg.pink,
                        target,
                        colors.fg.lightgrey,
                        colors.bold,
                        colors.fg.green,
                        "[...]" + data[-135:],
                        colors.reset,
                    )
                )
            else:
                print(
                    "{}{}{:15}{}|{}{:>25.25}{} > {}{}{}{}".format(
                        colors.fg.lightblue,
                        colors.bold,
                        source,
                        colors.fg.lightgrey,
                        colors.fg.pink,
                        target,
                        colors.fg.lightgrey,
                        colors.bold,
                        colors.fg.green,
                        data,
                        colors.reset,
                    )
                )
        # Underscore to avoid coloring like a HASH
        elif "_HASH" in source:
            print(
                "{}{:15}{}|{}{:>25.25}{} > {}{}{}".format(
                    colors.fg.lightblue,
                    source,
                    colors.fg.lightgrey,
                    colors.fg.pink,
                    target,
                    colors.fg.lightgrey,
                    colors.fg.red,
                    data,
                    colors.reset,
                )
            )
        # Underscore to avoid coloring service with "email" in name
        elif "_EMAIL" in source:
            print(
                "{}{:15}{}|{}{:>25.25}{} > {}{}{}".format(
                    colors.fg.lightblue,
                    source,
                    colors.fg.lightgrey,
                    colors.fg.pink,
                    target,
                    colors.fg.lightgrey,
                    colors.fg.lightgrey,
                    data,
                    colors.reset,
                )
            )
        elif "USER" in source:
            print(
                "{}{:15}{}|{}{:>25.25}{} > {}{}{}".format(
                    colors.fg.lightblue,
                    source,
                    colors.fg.lightgrey,
                    colors.fg.pink,
                    target,
                    colors.fg.lightgrey,
                    colors.fg.lightcyan,
                    data,
                    colors.reset,
                )
            )
        elif "SOURCE" in source:
            print(
                "{}{:15}{}|{}{:>25.25}{} > {}{}{}\n".format(
                    colors.fg.lightblue,
                    source,
                    colors.fg.lightgrey,
                    colors.fg.pink,
                    target,
                    colors.fg.lightgrey,
                    colors.reset,
                    data,
                    colors.reset,
                )
            )
        elif "IP" in source:
            print(
                "{}{:15}{}|{}{:>25.25}{} > {}{}{}".format(
                    colors.fg.lightblue,
                    source,
                    colors.fg.lightgrey,
                    colors.fg.pink,
                    target,
                    colors.fg.lightgrey,
                    colors.fg.red,
                    data,
                    colors.reset,
                )
            )
        else:
            print(
                "{}{:15}{}|{}{:>25.25}{} > {}{}{}".format(
                    colors.fg.lightblue,
                    source,
                    colors.fg.lightgrey,
                    colors.fg.pink,
                    target,
                    colors.fg.lightgrey,
                    colors.fg.lightgrey,
                    data,
                    colors.reset,
                )
            )

    @staticmethod
    def print_res_header(target):
        """
        Print Breach result header
        """
        print(colors.bold, "{:_^90}\n".format(""), colors.reset)
        print(
            colors.bold
            + colors.fg.green
            + "[>] "
            + colors.reset
            + "Showing results for "
            + target
            + colors.reset
        )



# -*- coding: utf-8 -*-
from .colors import colors as c
import os


def gen_config_file():
    with open(
        os.path.join(os.getcwd(), "h8mail_config.ini"), "w", newline=""
    ) as dest_config:
        config = """[h8mail]
; h8mail will automatically detect present keys & launch services accordingly
; Uncomment to activate
;hunterio = 
;snusbase_token = 
;;weleakinfo_priv = 
;;weleakinfo_pub = 
;hibp = 
;leak-lookup_pub = 1bf94ff907f68d511de9a610a6ff9263
;leak-lookup_priv = 
;emailrep = 
;dehashed_email = 
;dehashed_key =
;intelx_key =
;intelx_maxfile = 10
"""
        dest_config.write(config)
        c.good_news(
            "Generated h8mail template configuration file ({})".format(
                os.path.join(os.getcwd(), "h8mail_config.ini")
            )
        )



# -*- coding: utf-8 -*-

from multiprocessing import Pool
from .classes import local_breach_target
from .colors import colors as c
from .localsearch import raw_in_count, progress
import gzip
import os
import sys
import signal


def progress_gzip(count):
    """
    Prints count without rewriting to stdout
    """
    sys.stdout.write("Lines checked:%i\r" % (count))
    sys.stdout.write("\033[K")


def gzip_worker(filepath, target_list):
    """
    Searches for every email from target_list in every line of filepath.
    Uses python native gzip lib to decompress file line by line.
    Archives with multiple files are read as long single files.
    Attempts to decode line using cp437. If it fails, catch and use raw data.
    """
    try:
        found_list = []
        size = os.stat(filepath).st_size
        with gzip.open(filepath, "r") as gzipfile:
            c.info_news(
                "Worker [{PID}] is searching for targets in {filepath} ({size:,.0f} MB)".format(
                    PID=os.getpid(), filepath=filepath, size=size / float(1 << 20)
                )
            )
            for cnt, line in enumerate(gzipfile):
                for t in target_list:
                    if t in str(line):
                        try:
                            decoded = str(line, "cp437")
                            found_list.append(
                                local_breach_target(t, filepath, cnt, decoded)
                            )
                            c.good_news(
                                f"Found occurrence [{filepath}] Line {cnt}: {decoded}"
                            )
                        except Exception as e:
                            c.bad_news(
                                f"Got a decoding error line {cnt} - file: {filepath}"
                            )
                            c.good_news(
                                f"Found occurrence [{filepath}] Line {cnt}: {line}"
                            )
                            found_list.append(
                                local_breach_target(t, filepath, cnt, str(line))
                            )
        return found_list
    except Exception as e:
        c.bad_news("Something went wrong with gzip worker")
        print(e)


def local_gzip_search(files_to_parse, target_list):
    """
    Receives list of all files to check for target_list.
    Starts a worker pool, one worker per file.
    Return list of local_breach_targets objects to be tranformed in target objects.
    """
    original_sigint_handler = signal.signal(signal.SIGINT, signal.SIG_IGN)
    pool = Pool()
    found_list = []
    signal.signal(signal.SIGINT, original_sigint_handler)
    # Launch
    try:
        async_results = [
            pool.apply_async(gzip_worker, args=(f, target_list))
            for i, f in enumerate(files_to_parse)
        ]
        for r in async_results:
            if r.get() is not None:
                found_list.extend(r.get(60))
    except KeyboardInterrupt:
        c.bad_news("Caught KeyboardInterrupt, terminating workers")
        pool.terminate()
    else:
        c.info_news("Terminating worker pool")
        pool.close()
    pool.join()
    return found_list


def local_search_single_gzip(files_to_parse, target_list):
    """
    Single process searching of every target_list emails, in every files_to_parse list.
    To be used when stability for big files is a priority
    Return list of local_breach_target objects to be tranformed in target objects
    """
    found_list = []
    for file_to_parse in files_to_parse:
        with gzip.open(file_to_parse, "r") as fp:
            size = os.stat(file_to_parse).st_size
            c.info_news(
                f"Searching for targets in {file_to_parse} ({size} bytes)"
            )
            for cnt, line in enumerate(fp):
                progress_gzip(cnt)
                for t in target_list:
                    if t in str(line):
                        try:
                            decoded = str(line, "cp437")
                            found_list.append(
                                local_breach_target(t, file_to_parse, cnt, decoded)
                            )
                            c.good_news(
                                f"Found occurrence [{file_to_parse}] Line {cnt}: {decoded}"
                            )
                        except:
                            c.bad_news(
                                f"Got a decoding error line {cnt} - file: {file_to_parse}"
                            )
                            c.good_news(
                                f"Found occurrence [{file_to_parse}] Line {cnt}: {line}"
                            )
                            found_list.append(
                                local_breach_target(t, file_to_parse, cnt, str(line))
                            )
    return found_list



# -*- coding: utf-8 -*-

import time
from .colors import colors as c


def print_summary(start_time, breached_targets):
    """
    Prints a padded table where each line is a target, and associated value is simplified to breached/not breached.
    If breached, shows len(t.data). Shown elements and total may differ as some elements of t.data are not outputted to stdout.
    """
    print("{:_^90}".format(""))
    print(
        "\n\n\n{:^32}".format(""),
        c.bold,
        c.underline,
        "Session Recap:",
        c.reset,
        "\n\n",
    )
    print("{:^40} | ".format("Target"), "{:^40}".format("Status"), c.reset)
    print("{:_^90}\n".format(""))
    for t in breached_targets:
        if t.pwned != 0:
            print(
                f"{t.target:^40} | ",
                c.fg.green,
                "{:^40}".format("Breach Found (" + str(t.pwned) + " elements)"),
                c.reset,
            )
        else:
            print(
                f"{t.target:^40} | ",
                c.fg.lightgrey,
                "{:^40}".format("Not Compromised"),
                c.reset,
            )
        print("{:_^90}\n".format(""))
    total_time = time.time() - start_time
    print("Execution time (seconds): ", c.fg.lightcyan, f"{total_time}", c.reset, "\n")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `h8mail` package."""


import unittest
import sys
import time
import tempfile
import shutil
import contextlib
import os
import tarfile
import gzip
import argparse
from h8mail.utils import run
from h8mail.utils import classes
from h8mail.utils import helpers
from h8mail.utils import localsearch
from h8mail.utils import localgzipsearch

def print_test_banner(testname):
    print("========================")
    print("========================")
    print("\tTESTING: "+testname)
    print("========================")
    print("========================")


def make_temp_directory():
    emails = """
    john.smith@gmail.com
    john.smith@gmail.com
    fijsdhkfnhqsdkf
    fdqfqsdff
    test@evilcorp.com
    notfound@email.com
    """
    creds = """
    john.smith@gmail.com:SecretPASS
    bloblfd
    fjsdkf,ds
    test@evilcorp.com:An0therSECRETpassw0rd
    ddqsdqs
    """
    temp_dir = tempfile.mkdtemp()
    try:
        fd_emails = open(os.path.join(temp_dir, "test-emails.txt"), "w")
        fd_emails.writelines(emails)
        fd_emails.close()
        fd_creds = open(os.path.join(temp_dir, "test-creds.txt"), "w")
        fd_creds.writelines(creds)
        fd_creds.close()
        tar = tarfile.open(os.path.join(temp_dir, "test-creds.tar.gz"), "w:gz")
        tar.add(os.path.join(temp_dir, "test-creds.txt"))
        tar.close()

        return temp_dir
    except Exception as e:
        print(e)


class TestH8mail(unittest.TestCase):
    """Tests for `h8mail` package."""

    def setUp(self):
        """Generating local files"""
        self.temp_dir = make_temp_directory()
        print("Created Temp Dir: " + self.temp_dir)
        print(os.listdir(self.temp_dir))
        self.filetargets = os.path.join(self.temp_dir, "test-emails.txt")
        self.filetxt = os.path.join(self.temp_dir, "test-creds.txt")
        self.filegz = os.path.join(self.temp_dir, "test-creds.tar.gz")
        print("Test files generated in : " + self.temp_dir)

        # a = open(self.filetxt, "r")
        # print(a.readlines())

    def tearDown(self):
        """Cleaning temp files"""
        print("Removing dir + content: " + self.temp_dir)
        shutil.rmtree(self.temp_dir)

    def test_000_simple(self):
        """Simple test"""
        run.print_banner()
        print_test_banner("VANILLA")

        user_args = run.parse_args(["-t", "test@example.com"])
        run.h8mail(user_args)

    def test_002_local_files_txt_gz(self):
        """Local file search Test"""
        run.print_banner()
        print_test_banner("TXT LOCAL")
        user_args_lb = run.parse_args(["-t", self.filetargets, "-lb", self.filetxt, "-sk"])
        run.h8mail(user_args_lb)
        print_test_banner("TXT LOCAL-SINGLFILE")
        user_args_lb = run.parse_args(["-t", self.filetargets, "-lb", self.filetxt, "-sk", "-sf"])
        run.h8mail(user_args_lb)

        run.print_banner()
        print_test_banner("GZ LOCAL")
        user_args_gz = run.parse_args(["-t", self.filetargets, "-gz", self.filegz, "-sk"])
        run.h8mail(user_args_gz)
        print_test_banner("GZ LOCAL-SINGLEFILE")
        user_args_gz = run.parse_args(["-t", self.filetargets, "-gz", self.filegz, "-sk", "-sf"])
        run.h8mail(user_args_gz)

    def test_003_url(self):
        run.print_banner()
        print_test_banner("URL-RAW")
        user_args_lb = run.parse_args(["-u", "https://raw.githubusercontent.com/khast3x/h8mail/master/tests/test_email.txt"])
        run.h8mail(user_args_lb)
        run.print_banner()
        print_test_banner("URL-MESSY")
        user_args_lb = run.parse_args(["-u", "https://raw.githubusercontent.com/khast3x/h8mail/master/tests/test_email.txt"])
        run.h8mail(user_args_lb)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .colors import colors as c
import re

def chase(target, user_args):
    """
    Takes the current target & returns adequate chase list
    to add to the current target list
    """
    new_targets = []
    chase_counter = 0
    if user_args.debug:
        print(c.fg.red, "\nCHASING DEBUG-----------")
        print(f"Hunting targets from {target.target}")
        print(f"Recursive Chase Stop is at {user_args.chase_limit}" + c.reset)
    if user_args.chase_limit > 0:
        for d in target.data:
            if len(d) != 2:
                continue

            if user_args.power_chase:
                if "RELATED" in d[0] or "EMAIL" in d[0]:
                    c.good_news(
                        "Chasing {new_target} as new target".format(new_target=d[1])
                    )
                    new_targets.append(d[1])
                else: # in case there is an email as a username
                    e = re.findall(r"[\w\.-]+@[\w\.-]+", d[1])
                    if e:
                        for email in e:
                            c.good_news(
                                "Chasing {new_target} as new target (found as pattern)".format(
                                    new_target=d[1]
                                )
                            )
                            new_targets.append(d[1])

            else:
                if "HUNTER_RELATED" in d[0]:
                    c.good_news(
                        "Chasing {new_target} as new target".format(new_target=d[1])
                    )
                    new_targets.append(d[1])
    return new_targets
